void setup() {
  exxonTable = loadTable("emt.csv");
  lockheedTable = loadTable("lmt.csv");
  spTable = loadTable("spt.csv");
  gsTable = loadTable("gst.csv");
  goldTable = loadTable("gold.csv");

  int numTotalRows = Math.max(lockheedTable.getRowCount(), exxonTable.getRowCount());
  PrintWriter output = createWriter("merged.csv"); 

  indexMap = new HashMap<String, List<Integer>>(numTotalRows);
  allDates=new HashSet<String>(numTotalRows);
  addTableToHash(lockheedTable);
  addTableToHash(exxonTable);
  addTableToHash(spTable);
  addTableToHash(gsTable);
  addTableToHash(goldTable);


  Set<String> keys = indexMap.keySet();
  String[] dates = indexMap.keySet().toArray(new String[numTotalRows]);
  HashMap<String, Float> lockheedMap = new HashMap<String, Float>();
  HashMap<String, Float> exxonMap = new HashMap<String, Float>();
  HashMap<String, Float> sanPMap = new HashMap<String, Float>();
  HashMap<String, Float> gsMap = new HashMap<String, Float>();
  HashMap<String, Float> goldMap = new HashMap<String, Float>();


  dateSort(dates);
  for (int i = 0; i < lockheedTable.getRowCount (); i++) {
    lockheedMap.put(lockheedTable.getString(i, 0), lockheedTable.getFloat(i, 1));
  }

  for (int i = 0; i < exxonTable.getRowCount (); i++) {
    exxonMap.put(exxonTable.getString(i, 0), exxonTable.getFloat(i, 1));
  }

  for (int i = 0; i < spTable.getRowCount (); i++) {
    sanPMap.put(spTable.getString(i, 0), spTable.getFloat(i, 1));
  } 

  for (int i = 0; i < gsTable.getRowCount (); i++) {
    gsMap.put(gsTable.getString(i, 0), gsTable.getFloat(i, 1));
  } 

  for (int i = 0; i < goldTable.getRowCount (); i++) {
    goldMap.put(goldTable.getString(i, 0), goldTable.getFloat(i, 1));
  } 


  output.println("Date, lockheed, exxon, S&P, goldman, gold");
  for (int i = 0; i < dates.length; i++) {
    float lVal = -1;
    float eVal = -1;
    float sVal = -1;
    float gVal = -1;
    float auVal = -1;


    if (lockheedMap.get(dates[i])!=null) {
      lVal = lockheedMap.get(dates[i]);
    } 

    if (exxonMap.get(dates[i])!=null) {
      eVal = exxonMap.get(dates[i]);
    } 
    if (sanPMap.get(dates[i])!=null) {
      sVal = sanPMap.get(dates[i]);
    } 

    if (gsMap.get(dates[i])!=null) {
      gVal = gsMap.get(dates[i]);
    } 

    if (goldMap.get(dates[i])!=null) {
      auVal = goldMap.get(dates[i]);
    } 
    output.println(dates[i] +", " + lVal +", " +eVal + ", " + sVal + ", " + gVal + ", " + auVal);
  }
  output.flush();
  println("done");
}




void addTableToHash(Table table) {
  for (int i = 1; i < table.getRowCount (); i++) {
    List<Integer> date = indexMap.get(table.getString(i, 0));
    if (date == null) {
      date = new ArrayList<Integer>();
      date.add(i);
      indexMap.put(table.getString(i, 0), date);
    } else {
      date.add(i);
      indexMap.put(table.getString(i, 0), date);
    }
  }
}

void addTableToIndexMap(Table table) {
  //    for (int i = 1; i < table.getRowCount (); i++) {
  //    List<Integer> date = indexMap.get(table.getString(i, 0));
  //    if (date == null) {
  //      date = new ArrayList<Integer>();
  //      date.add(i);
  //      indexMap.put(table.getString(i, 0), table.getFloat(i,1));
  //    } else {
  //      date.add(i);
  //      indexMap.put(table.getString(i, 0), table.getFloat(i,1));
  //    }
  //  }
}

String[] dateSort(String [] dates) {
  int len = dates.length;
  while (len!=0) {
    int nlen = 0;
    for (int i = 1; i <= len - 1; i++) {
      if (compare(dates[i-1], dates[i]) ==1) {
        String tmp = dates[i-1];
        dates[i-1] = dates[i];
        dates[i]=tmp;
        nlen = i;
      }
    }
    len = nlen;
  }
  return dates;
}

int compare(String a, String b) {
  String[] aParse = a.split("/");
  String[] bParse = b.split("/");
  if (Integer.parseInt(aParse[2])>Integer.parseInt(bParse[2])) return 1;
  else if (Integer.parseInt(aParse[2])==Integer.parseInt(bParse[2])) {
    if (Integer.parseInt(aParse[0])>Integer.parseInt(bParse[0])) return 1; 
    else if (Integer.parseInt(aParse[0])==Integer.parseInt(bParse[0])) {
      if (Integer.parseInt(aParse[1])>Integer.parseInt(bParse[1])) return 1;
    }
  }
  if (Integer.parseInt(aParse[2])<Integer.parseInt(bParse[2])) return -1;
  else if (Integer.parseInt(aParse[2])==Integer.parseInt(bParse[2])) {
    if (Integer.parseInt(aParse[0])<Integer.parseInt(bParse[0])) return -1; 
    else if (Integer.parseInt(aParse[0])==Integer.parseInt(bParse[0])) {
      if (Integer.parseInt(aParse[1])<Integer.parseInt(bParse[1])) return -1;
    }
  }


  return 0;
}

